@@ -47,40 +47,52 @@
 
 /**
  * SECTION:buffer
- * @Short_description: Buffer object for GtkSourceView
+ * @Short_description: Stores the text for display in a GtkSourceView
  * @Title: GtkSourceBuffer
  * @See_also: #GtkTextBuffer, #GtkSourceView
  *
- * The #GtkSourceBuffer object is the model for #GtkSourceView widgets.
- * It extends the #GtkTextBuffer object by adding features useful to display
+ * A #GtkSourceBuffer object is the model for #GtkSourceView widgets.
+ * It extends the #GtkTextBuffer class by adding features useful to display
  * and edit source code such as syntax highlighting and bracket matching. It
- * also implements support for undo/redo operations, and for the search and
- * replace.
+ * also implements support for the undo/redo.
  *
  * To create a #GtkSourceBuffer use gtk_source_buffer_new() or
  * gtk_source_buffer_new_with_language(). The second form is just a convenience
- * function which allows you to initially set a #GtkSourceLanguage.
+ * function which allows you to initially set a #GtkSourceLanguage. You can also
+ * directly create a #GtkSourceView and get its #GtkSourceBuffer with
+ * gtk_text_view_get_buffer().
  *
- * By default highlighting is enabled, but you can disable it with
+ * The highlighting is enabled by default, but you can disable it with
  * gtk_source_buffer_set_highlight_syntax().
  *
- * # Undo and Redo
+ * # Undo/Redo
  *
  * A custom #GtkSourceUndoManager can be implemented and set with
  * gtk_source_buffer_set_undo_manager(). However the default implementation
- * should be suitable for most uses. By default, actions that can be undone or
- * redone are defined as groups of operations between a call to
- * gtk_text_buffer_begin_user_action() and gtk_text_buffer_end_user_action(). In
- * general, this happens whenever the user presses any key which modifies the
- * buffer. But the default undo manager will try to merge similar consecutive
- * actions, such as multiple character insertions on the same line, into one
- * action. But, inserting a newline starts a new action.
+ * should be suitable for most uses, so you can use the API provided by
+ * #GtkSourceBuffer instead of using #GtkSourceUndoManager. By default, actions
+ * that can be undone or redone are defined as groups of operations between a
+ * call to gtk_text_buffer_begin_user_action() and
+ * gtk_text_buffer_end_user_action(). In general, this happens whenever the user
+ * presses any key which modifies the buffer. But the default undo manager will
+ * try to merge similar consecutive actions into one undo/redo level. The
+ * merging is done word by word, so after writing a new sentence (character by
+ * character), each undo will remove the previous word.
  *
  * The default undo manager remembers the "modified" state of the buffer, and
- * restore it when an action is undone or redone. It can be useful in a text
+ * restores it when an action is undone or redone. It can be useful in a text
  * editor to know whether the file is saved. See gtk_text_buffer_get_modified()
  * and gtk_text_buffer_set_modified().
  *
+ * The default undo manager also restores the selected text (or cursor
+ * position), if the selection was related to the action. For example if the
+ * user selects some text and deletes it, an undo will restore the selection. On
+ * the other hand, if some text is selected but a deletion occurs elsewhere (the
+ * deletion was done programmatically), an undo will not restore the selection,
+ * it will only moves the cursor (the cursor is moved so that the user sees the
+ * undo's effect). Warning: the selection restoring behavior might change in the
+ * future.
+ *
  * # Context Classes
  *
  * It is possible to retrieve some information from the syntax highlighting
@@ -2298,87 +2310,158 @@ gtk_source_buffer_iter_backward_to_context_class_toggle (GtkSourceBuffer *buffer
 	}
 }
 
-static void
-do_lower_case (GtkTextBuffer *buffer,
-               GtkTextIter   *start,
-               GtkTextIter   *end,
-               GString       *str)
+static gchar *
+do_lower_case (GtkTextBuffer     *buffer,
+	       const GtkTextIter *start,
+	       const GtkTextIter *end)
 {
-	while (!gtk_text_iter_is_end (start) &&
-	       !gtk_text_iter_equal (start, end))
-	{
-		gunichar c, nc;
+	gchar *text;
+	gchar *new_text;
 
-		c = gtk_text_iter_get_char (start);
-		nc = g_unichar_tolower (c);
-		g_string_append_unichar (str, nc);
+	text = gtk_text_buffer_get_text (buffer, start, end, TRUE);
+	new_text = g_utf8_strdown (text, -1);
 
-		gtk_text_iter_forward_char (start);
-	}
+	g_free (text);
+	return new_text;
 }
 
-static void
-do_upper_case (GtkTextBuffer *buffer,
-               GtkTextIter   *start,
-               GtkTextIter   *end,
-               GString       *str)
+static gchar *
+do_upper_case (GtkTextBuffer     *buffer,
+	       const GtkTextIter *start,
+	       const GtkTextIter *end)
 {
-	while (!gtk_text_iter_is_end (start) &&
-	       !gtk_text_iter_equal (start, end))
-	{
-		gunichar c, nc;
+	gchar *text;
+	gchar *new_text;
 
-		c = gtk_text_iter_get_char (start);
-		nc = g_unichar_toupper (c);
-		g_string_append_unichar (str, nc);
+	text = gtk_text_buffer_get_text (buffer, start, end, TRUE);
+	new_text = g_utf8_strup (text, -1);
 
-		gtk_text_iter_forward_char (start);
-	}
+	g_free (text);
+	return new_text;
 }
 
-static void
-do_toggle_case (GtkTextBuffer *buffer,
-                GtkTextIter   *start,
-                GtkTextIter   *end,
-                GString       *str)
+static gchar *
+do_toggle_case (GtkTextBuffer     *buffer,
+		const GtkTextIter *start,
+		const GtkTextIter *end)
 {
-	while (!gtk_text_iter_is_end (start) &&
-	       !gtk_text_iter_equal (start, end))
-	{
-		gunichar c, nc;
+	GString *str;
+	GtkTextIter iter_start;
 
-		c = gtk_text_iter_get_char (start);
-		if (g_unichar_islower (c))
-			nc = g_unichar_toupper (c);
+	str = g_string_new (NULL);
+	iter_start = *start;
+
+	while (!gtk_text_iter_is_end (&iter_start))
+	{
+		GtkTextIter iter_end;
+		gchar *text;
+		gchar *text_down;
+		gchar *text_up;
+
+		iter_end = iter_start;
+		gtk_text_iter_forward_cursor_position (&iter_end);
+
+		if (gtk_text_iter_compare (end, &iter_end) < 0)
+		{
+			break;
+		}
+
+		text = gtk_text_buffer_get_text (buffer, &iter_start, &iter_end, TRUE);
+		text_down = g_utf8_strdown (text, -1);
+		text_up = g_utf8_strup (text, -1);
+
+		if (g_strcmp0 (text, text_down) == 0)
+		{
+			g_string_append (str, text_up);
+		}
+		else if (g_strcmp0 (text, text_up) == 0)
+		{
+			g_string_append (str, text_down);
+		}
 		else
-			nc = g_unichar_tolower (c);
-		g_string_append_unichar (str, nc);
+		{
+			g_string_append (str, text);
+		}
 
-		gtk_text_iter_forward_char (start);
+		g_free (text);
+		g_free (text_down);
+		g_free (text_up);
+
+		iter_start = iter_end;
 	}
+
+	return g_string_free (str, FALSE);
 }
 
-static void
-do_title_case (GtkTextBuffer *buffer,
-               GtkTextIter   *start,
-               GtkTextIter   *end,
-               GString       *str)
+static gchar *
+do_title_case (GtkTextBuffer     *buffer,
+	       const GtkTextIter *start,
+	       const GtkTextIter *end)
 {
+	GString *str;
+	GtkTextIter iter_start;
 
-	while (!gtk_text_iter_is_end (start) &&
-	       !gtk_text_iter_equal (start, end))
+	str = g_string_new (NULL);
+	iter_start = *start;
+
+	while (!gtk_text_iter_is_end (&iter_start))
 	{
-		gunichar c, nc;
+		GtkTextIter iter_end;
+		gchar *text;
 
-		c = gtk_text_iter_get_char (start);
-		if (gtk_text_iter_starts_word (start))
-			nc = g_unichar_totitle (c);
+		iter_end = iter_start;
+		gtk_text_iter_forward_cursor_position (&iter_end);
+
+		if (gtk_text_iter_compare (end, &iter_end) < 0)
+		{
+			break;
+		}
+
+		text = gtk_text_buffer_get_text (buffer, &iter_start, &iter_end, TRUE);
+
+		if (gtk_text_iter_starts_word (&iter_start))
+		{
+			gchar *text_normalized;
+
+			text_normalized = g_utf8_normalize (text, -1, G_NORMALIZE_DEFAULT);
+
+			if (g_utf8_strlen (text_normalized, -1) == 1)
+			{
+				gunichar c;
+				gunichar new_c;
+
+				c = gtk_text_iter_get_char (&iter_start);
+				new_c = g_unichar_totitle (c);
+
+				g_string_append_unichar (str, new_c);
+			}
+			else
+			{
+				gchar *text_up;
+
+				text_up = g_utf8_strup (text, -1);
+				g_string_append (str, text_up);
+
+				g_free (text_up);
+			}
+
+			g_free (text_normalized);
+		}
 		else
-			nc = g_unichar_tolower (c);
-		g_string_append_unichar (str, nc);
+		{
+			gchar *text_down;
 
-		gtk_text_iter_forward_char (start);
+			text_down = g_utf8_strdown (text, -1);
+			g_string_append (str, text_down);
+
+			g_free (text_down);
+		}
+
+		g_free (text);
+		iter_start = iter_end;
 	}
+
+	return g_string_free (str, FALSE);
 }
 
 /**
@@ -2398,40 +2481,45 @@ gtk_source_buffer_change_case (GtkSourceBuffer         *buffer,
                                GtkTextIter             *start,
                                GtkTextIter             *end)
 {
-	GtkTextIter i;
-	GString *s;
+	GtkTextBuffer *text_buffer;
+	gchar *new_text;
 
 	g_return_if_fail (GTK_SOURCE_IS_BUFFER (buffer));
 	g_return_if_fail (start != NULL);
 	g_return_if_fail (end != NULL);
 
-	i = *start;
-	s = g_string_new (NULL);
+	gtk_text_iter_order (start, end);
+
+	text_buffer = GTK_TEXT_BUFFER (buffer);
 
 	switch (case_type)
 	{
-	case GTK_SOURCE_CHANGE_CASE_UPPER:
-		do_upper_case (GTK_TEXT_BUFFER (buffer), &i, end, s);
-		break;
-	case GTK_SOURCE_CHANGE_CASE_LOWER:
-		do_lower_case (GTK_TEXT_BUFFER (buffer), &i, end, s);
-		break;
-	case GTK_SOURCE_CHANGE_CASE_TOGGLE:
-		do_toggle_case (GTK_TEXT_BUFFER (buffer), &i, end, s);
-		break;
-	case GTK_SOURCE_CHANGE_CASE_TITLE:
-		do_title_case (GTK_TEXT_BUFFER (buffer), &i, end, s);
-		break;
-	default:
-		g_return_if_reached ();
+		case GTK_SOURCE_CHANGE_CASE_LOWER:
+			new_text = do_lower_case (text_buffer, start, end);
+			break;
+
+		case GTK_SOURCE_CHANGE_CASE_UPPER:
+			new_text = do_upper_case (text_buffer, start, end);
+			break;
+
+		case GTK_SOURCE_CHANGE_CASE_TOGGLE:
+			new_text = do_toggle_case (text_buffer, start, end);
+			break;
+
+		case GTK_SOURCE_CHANGE_CASE_TITLE:
+			new_text = do_title_case (text_buffer, start, end);
+			break;
+
+		default:
+			g_return_if_reached ();
 	}
 
-	gtk_text_buffer_begin_user_action (GTK_TEXT_BUFFER (buffer));
-	gtk_text_buffer_delete_interactive (GTK_TEXT_BUFFER (buffer), start, end, TRUE);
-	gtk_text_buffer_insert_interactive (GTK_TEXT_BUFFER (buffer), start, s->str, s->len, TRUE);
-	gtk_text_buffer_end_user_action (GTK_TEXT_BUFFER (buffer));
+	gtk_text_buffer_begin_user_action (text_buffer);
+	gtk_text_buffer_delete (text_buffer, start, end);
+	gtk_text_buffer_insert (text_buffer, start, new_text, -1);
+	gtk_text_buffer_end_user_action (text_buffer);
 
-	g_string_free (s, TRUE);
+	g_free (new_text);
 }
 
 /**
