@@ -353,8 +353,8 @@ do_redraw (GtkSourceGutter *gutter)
 }
 
 static gint
-calculate_gutter_size (GtkSourceGutter  *gutter,
-                       GArray           *sizes)
+calculate_gutter_size (GtkSourceGutter *gutter,
+		       GArray          *sizes)
 {
 	GList *item;
 	gint total_width = 0;
@@ -538,8 +538,8 @@ gtk_source_gutter_class_init (GtkSourceGutterClass *klass)
 	 *
 	 * The x-padding.
 	 *
-	 * Deprecated: 3.12: Use the #GtkSourceGutterRenderer:xpad property
-	 * instead.
+	 * Deprecated: 3.12: Use the #GtkSourceGutterRenderer's
+	 * #GtkSourceGutterRenderer:xpad property instead.
 	 */
 	g_object_class_install_property (object_class,
 	                                 PROP_XPAD,
@@ -549,15 +549,17 @@ gtk_source_gutter_class_init (GtkSourceGutterClass *klass)
 	                                                   -1,
 	                                                   G_MAXINT,
 	                                                   0,
-	                                                   G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+	                                                   G_PARAM_READWRITE |
+							   G_PARAM_CONSTRUCT |
+							   G_PARAM_DEPRECATED));
 
 	/**
 	 * GtkSourceGutter:ypad:
 	 *
 	 * The y-padding.
 	 *
-	 * Deprecated: 3.12: Use the #GtkSourceGutterRenderer:ypad property
-	 * instead.
+	 * Deprecated: 3.12: Use the #GtkSourceGutterRenderer's
+	 * #GtkSourceGutterRenderer:ypad property instead.
 	 */
 	g_object_class_install_property (object_class,
 	                                 PROP_YPAD,
@@ -567,7 +569,9 @@ gtk_source_gutter_class_init (GtkSourceGutterClass *klass)
 	                                                   -1,
 	                                                   G_MAXINT,
 	                                                   0,
-	                                                   G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+	                                                   G_PARAM_READWRITE |
+							   G_PARAM_CONSTRUCT |
+							   G_PARAM_DEPRECATED));
 }
 
 static void
@@ -780,61 +784,87 @@ gtk_source_gutter_queue_draw (GtkSourceGutter *gutter)
 	do_redraw (gutter);
 }
 
-/* This function is taken from gtk+/tests/testtext.c */
-static gint
-get_lines (GtkTextView  *text_view,
-           gint          first_y,
-           gint          last_y,
-           GArray       *buffer_coords,
-           GArray       *line_heights,
-           GArray       *numbers,
-           gint         *countp,
-           GtkTextIter  *start,
-           GtkTextIter  *end)
+typedef struct _LinesInfo LinesInfo;
+
+struct _LinesInfo
 {
-	GtkTextIter iter;
-	gint count;
-	gint last_line_num = -1;
-	gint total_height = 0;
+	gint total_height;
+	gint lines_count;
+	GArray *buffer_coords;
+	GArray *line_heights;
+	GArray *line_numbers;
+	GtkTextIter start;
+	GtkTextIter end;
+};
 
-	g_array_set_size (buffer_coords, 0);
-	g_array_set_size (numbers, 0);
+static LinesInfo *
+lines_info_new (void)
+{
+	LinesInfo *info;
 
-	if (line_heights != NULL)
+	info = g_slice_new0 (LinesInfo);
+
+	info->buffer_coords = g_array_new (FALSE, FALSE, sizeof (gint));
+	info->line_heights = g_array_new (FALSE, FALSE, sizeof (gint));
+	info->line_numbers = g_array_new (FALSE, FALSE, sizeof (gint));
+
+	return info;
+}
+
+static void
+lines_info_free (LinesInfo *info)
+{
+	if (info != NULL)
 	{
-		g_array_set_size (line_heights, 0);
+		g_array_free (info->buffer_coords, TRUE);
+		g_array_free (info->line_heights, TRUE);
+		g_array_free (info->line_numbers, TRUE);
+
+		g_slice_free (LinesInfo, info);
 	}
+}
+
+/* This function is taken and adapted from gtk+/tests/testtext.c */
+static LinesInfo *
+get_lines_info (GtkTextView *text_view,
+		gint         first_y_buffer_coord,
+		gint         last_y_buffer_coord)
+{
+	LinesInfo *info;
+	GtkTextIter iter;
+	gint last_line_num = -1;
+
+	info = lines_info_new ();
 
 	/* Get iter at first y */
-	gtk_text_view_get_line_at_y (text_view, &iter, first_y, NULL);
+	gtk_text_view_get_line_at_y (text_view, &iter, first_y_buffer_coord, NULL);
+
+	info->start = iter;
 
 	/* For each iter, get its location and add it to the arrays.
-	 * Stop when we pass last_y */
-	count = 0;
-
-	*start = iter;
-
+	 * Stop when we pass last_y_buffer_coord.
+	 */
 	while (!gtk_text_iter_is_end (&iter))
 	{
-		gint y, height;
+		gint y;
+		gint height;
+		gint line_num;
 
 		gtk_text_view_get_line_yrange (text_view, &iter, &y, &height);
 
-		g_array_append_val (buffer_coords, y);
+		g_array_append_val (info->buffer_coords, y);
+		g_array_append_val (info->line_heights, height);
 
-		if (line_heights)
-		{
-			g_array_append_val (line_heights, height);
-		}
+		info->total_height += height;
 
-		total_height += height;
+		line_num = gtk_text_iter_get_line (&iter);
+		g_array_append_val (info->line_numbers, line_num);
 
-		last_line_num = gtk_text_iter_get_line (&iter);
-		g_array_append_val (numbers, last_line_num);
+		last_line_num = line_num;
 
-		++count;
+		info->lines_count++;
 
-		if ((y + height) >= last_y)
+		if (last_y_buffer_coord <= (y + height))
 		{
 			break;
 		}
@@ -844,7 +874,8 @@ get_lines (GtkTextView  *text_view,
 
 	if (gtk_text_iter_is_end (&iter))
 	{
-		gint y, height;
+		gint y;
+		gint height;
 		gint line_num;
 
 		gtk_text_view_get_line_yrange (text_view, &iter, &y, &height);
@@ -853,79 +884,46 @@ get_lines (GtkTextView  *text_view,
 
 		if (line_num != last_line_num)
 		{
-			g_array_append_val (buffer_coords, y);
+			g_array_append_val (info->buffer_coords, y);
+			g_array_append_val (info->line_heights, height);
 
-			if (line_heights)
-			{
-				g_array_append_val (line_heights, height);
-			}
+			info->total_height += height;
 
-			total_height += height;
-
-			g_array_append_val (numbers, line_num);
-			++count;
+			g_array_append_val (info->line_numbers, line_num);
+			info->lines_count++;
 		}
 	}
 
-	*countp = count;
-
-	if (count == 0)
+	if (info->lines_count == 0)
 	{
 		gint y = 0;
 		gint n = 0;
 		gint height;
 
-		*countp = 1;
+		info->lines_count = 1;
 
-		g_array_append_val (buffer_coords, y);
-		g_array_append_val (numbers, n);
+		g_array_append_val (info->buffer_coords, y);
+		g_array_append_val (info->line_numbers, n);
 
-		if (line_heights)
-		{
-			gtk_text_view_get_line_yrange (text_view, &iter, &y, &height);
-			g_array_append_val (line_heights, height);
+		gtk_text_view_get_line_yrange (text_view, &iter, &y, &height);
+		g_array_append_val (info->line_heights, height);
 
-			total_height += height;
-		}
+		info->total_height += height;
 	}
 
-	*end = iter;
+	info->end = iter;
 
-	return total_height;
+	return info;
 }
 
+/* Returns %TRUE if @clip is set. @clip contains the area that should be drawn. */
 static gboolean
-on_view_draw (GtkSourceView   *view,
-              cairo_t         *cr,
-              GtkSourceGutter *gutter)
+get_clip_rectangle (GtkSourceGutter *gutter,
+		    GtkSourceView   *view,
+		    cairo_t         *cr,
+		    GdkRectangle    *clip)
 {
-	GdkWindow *window;
-	GtkTextView *text_view;
-	GArray *sizes;
-	GdkRectangle clip;
-	gint y1, y2;
-	GArray *numbers;
-	GArray *pixels;
-	GArray *heights;
-	GtkTextIter cur;
-	gint cur_line;
-	gint count;
-	gint i;
-	GList *item;
-	GtkTextIter start;
-	GtkTextIter end;
-	GtkTextBuffer *buffer;
-	GdkRectangle background_area;
-	GdkRectangle cell_area;
-	GtkTextIter selection_start;
-	GtkTextIter selection_end;
-	gboolean has_selection;
-	gint idx;
-	GtkStyleContext *style_context;
-	const gchar *class;
-	GdkRGBA fg_color;
-
-	window = get_window (gutter);
+	GdkWindow *window = get_window (gutter);
 
 	if (window == NULL || !gtk_cairo_should_draw_window (cr, window))
 	{
@@ -934,88 +932,43 @@ on_view_draw (GtkSourceView   *view,
 
 	gtk_cairo_transform_to_window (cr, GTK_WIDGET (view), window);
 
-	if (!gdk_cairo_get_clip_rectangle (cr, &clip))
-	{
-		return FALSE;
-	}
+	return gdk_cairo_get_clip_rectangle (cr, clip);
+}
 
-	gutter->priv->is_drawing = TRUE;
-
-	text_view = GTK_TEXT_VIEW (view);
-	buffer = gtk_text_view_get_buffer (text_view);
-
-	y1 = clip.y;
-	y2 = y1 + clip.height;
-
-	/* get the extents of the line printing */
-	gtk_text_view_window_to_buffer_coords (text_view,
-	                                       gutter->priv->window_type,
-	                                       0,
-	                                       y1,
-	                                       NULL,
-	                                       &y1);
-
-	gtk_text_view_window_to_buffer_coords (text_view,
-	                                       gutter->priv->window_type,
-	                                       0,
-	                                       y2,
-	                                       NULL,
-	                                       &y2);
-
-	numbers = g_array_new (FALSE, FALSE, sizeof (gint));
-	pixels = g_array_new (FALSE, FALSE, sizeof (gint));
-	heights = g_array_new (FALSE, FALSE, sizeof (gint));
-	sizes = g_array_new (FALSE, FALSE, sizeof (gint));
-
-	calculate_gutter_size (gutter, sizes);
-
-	background_area.x = 0;
-	background_area.height = get_lines (text_view,
-	                                    y1,
-	                                    y2,
-	                                    pixels,
-	                                    heights,
-	                                    numbers,
-	                                    &count,
-	                                    &start,
-	                                    &end);
-
-	cell_area.x = 0;
-	cell_area.height = background_area.height;
-
-	gtk_text_view_buffer_to_window_coords (text_view,
-	                                       gutter->priv->window_type,
-	                                       0,
-	                                       g_array_index (pixels, gint, 0),
-	                                       NULL,
-	                                       &background_area.y);
-
-	cell_area.y = background_area.y;
-
-	style_context = gtk_widget_get_style_context (GTK_WIDGET (view));
-
-	gtk_style_context_save (style_context);
+static void
+apply_style (GtkSourceGutter *gutter,
+	     GtkSourceView   *view,
+	     GtkStyleContext *style_context,
+	     cairo_t         *cr)
+{
+	const gchar *class;
+	GdkRGBA fg_color;
 
 	switch (gutter->priv->window_type)
 	{
 		case GTK_TEXT_WINDOW_TOP:
 			class = GTK_STYLE_CLASS_TOP;
 			break;
+
 		case GTK_TEXT_WINDOW_RIGHT:
 			class = GTK_STYLE_CLASS_RIGHT;
 			break;
+
 		case GTK_TEXT_WINDOW_BOTTOM:
 			class = GTK_STYLE_CLASS_BOTTOM;
 			break;
+
 		case GTK_TEXT_WINDOW_LEFT:
 			class = GTK_STYLE_CLASS_LEFT;
 			break;
+
 		default:
-			g_return_val_if_reached (FALSE);
+			g_return_if_reached ();
 	}
 
 	/* Apply classes ourselves, since we are in connect_after and so they
-	 * are not set by gtk */
+	 * are not set by gtk.
+	 */
 	gtk_style_context_add_class (style_context, class);
 
 	gtk_style_context_get_color (style_context,
@@ -1023,51 +976,99 @@ on_view_draw (GtkSourceView   *view,
 	                             &fg_color);
 
 	gdk_cairo_set_source_rgba (cr, &fg_color);
+}
 
-	for (item = gutter->priv->renderers, idx = 0;
-	     item != NULL;
-	     item = g_list_next (item), idx++)
+/* Call gtk_source_gutter_renderer_begin() on each renderer. */
+static void
+begin_draw (GtkSourceGutter *gutter,
+	    GtkTextView     *view,
+	    GArray          *renderer_widths,
+	    LinesInfo       *info,
+	    cairo_t         *cr)
+{
+	GdkRectangle background_area;
+	GdkRectangle cell_area;
+	GList *l;
+	gint renderer_num;
+
+	background_area.x = 0;
+	background_area.height = info->total_height;
+
+	gtk_text_view_buffer_to_window_coords (view,
+	                                       gutter->priv->window_type,
+	                                       0,
+	                                       g_array_index (info->buffer_coords, gint, 0),
+	                                       NULL,
+	                                       &background_area.y);
+
+	cell_area = background_area;
+
+	for (l = gutter->priv->renderers, renderer_num = 0;
+	     l != NULL;
+	     l = l->next, renderer_num++)
 	{
-		Renderer *renderer = item->data;
-		gint xpad;
+		Renderer *renderer = l->data;
 		gint width;
+		gint xpad;
 
-		width = g_array_index (sizes, gint, idx);
+		width = g_array_index (renderer_widths, gint, renderer_num);
 
-		if (gtk_source_gutter_renderer_get_visible (renderer->renderer))
+		if (!gtk_source_gutter_renderer_get_visible (renderer->renderer))
 		{
-			gtk_source_gutter_renderer_get_padding (renderer->renderer,
-			                                        &xpad,
-			                                        NULL);
-
-			background_area.width = width;
-
-			cell_area.width = width - 2 * xpad;
-			cell_area.x = background_area.x + xpad;
-
-			cairo_save (cr);
-
-			gdk_cairo_rectangle (cr, &background_area);
-			cairo_clip (cr);
-
-			gtk_source_gutter_renderer_begin (renderer->renderer,
-			                                  cr,
-			                                  &background_area,
-			                                  &cell_area,
-			                                  &start,
-			                                  &end);
-
-			cairo_restore (cr);
-
-			background_area.x += background_area.width;
+			g_assert_cmpint (width, ==, 0);
+			continue;
 		}
+
+		gtk_source_gutter_renderer_get_padding (renderer->renderer,
+							&xpad,
+							NULL);
+
+		background_area.width = width;
+
+		cell_area.width = background_area.width - 2 * xpad;
+		cell_area.x = background_area.x + xpad;
+
+		cairo_save (cr);
+
+		gdk_cairo_rectangle (cr, &background_area);
+		cairo_clip (cr);
+
+		gtk_source_gutter_renderer_begin (renderer->renderer,
+						  cr,
+						  &background_area,
+						  &cell_area,
+						  &info->start,
+						  &info->end);
+
+		cairo_restore (cr);
+
+		background_area.x += background_area.width;
 	}
+}
+
+static void
+draw_cells (GtkSourceGutter *gutter,
+	    GtkTextView     *view,
+	    GArray          *renderer_widths,
+	    LinesInfo       *info,
+	    cairo_t         *cr)
+{
+	GtkTextBuffer *buffer;
+	GtkTextIter insert_iter;
+	gint cur_line;
+	GtkTextIter selection_start;
+	GtkTextIter selection_end;
+	gboolean has_selection;
+	GtkTextIter start;
+	gint i;
+
+	buffer = gtk_text_view_get_buffer (view);
 
 	gtk_text_buffer_get_iter_at_mark (buffer,
-	                                  &cur,
+	                                  &insert_iter,
 	                                  gtk_text_buffer_get_insert (buffer));
 
-	cur_line = gtk_text_iter_get_line (&cur);
+	cur_line = gtk_text_iter_get_line (&insert_iter);
 
 	gtk_text_buffer_get_selection_bounds (buffer,
 	                                      &selection_start,
@@ -1088,10 +1089,18 @@ on_view_draw (GtkSourceView   *view,
 		}
 	}
 
-	for (i = 0; i < count; ++i)
+	start = info->start;
+	i = 0;
+
+	while (i < info->lines_count)
 	{
+		GtkTextIter end;
+		GdkRectangle background_area;
+		GtkSourceGutterRendererState state;
 		gint pos;
 		gint line_to_paint;
+		gint renderer_num;
+		GList *l;
 
 		end = start;
 
@@ -1100,34 +1109,53 @@ on_view_draw (GtkSourceView   *view,
 			gtk_text_iter_forward_to_line_end (&end);
 		}
 
-		gtk_text_view_buffer_to_window_coords (text_view,
+		/* Possible improvement: if buffer and window coords have the
+		 * same unit, there are probably some possible performance
+		 * improvements by avoiding some buffer <-> window coords
+		 * conversions.
+		 */
+		gtk_text_view_buffer_to_window_coords (view,
 		                                       gutter->priv->window_type,
 		                                       0,
-		                                       g_array_index (pixels, gint, i),
+		                                       g_array_index (info->buffer_coords, gint, i),
 		                                       NULL,
 		                                       &pos);
 
-		line_to_paint = g_array_index (numbers, gint, i);
+		line_to_paint = g_array_index (info->line_numbers, gint, i);
 
 		background_area.y = pos;
-		background_area.height = g_array_index (heights, gint, i);
+		background_area.height = g_array_index (info->line_heights, gint, i);
 		background_area.x = 0;
 
-		for (item = gutter->priv->renderers, idx = 0;
-		     item != NULL;
-		     item = g_list_next (item), idx++)
+		state = GTK_SOURCE_GUTTER_RENDERER_STATE_NORMAL;
+
+		if (line_to_paint == cur_line)
+		{
+			state |= GTK_SOURCE_GUTTER_RENDERER_STATE_CURSOR;
+		}
+
+		if (has_selection &&
+		    gtk_text_iter_in_range (&start, &selection_start, &selection_end))
+		{
+			state |= GTK_SOURCE_GUTTER_RENDERER_STATE_SELECTED;
+		}
+
+		for (l = gutter->priv->renderers, renderer_num = 0;
+		     l != NULL;
+		     l = l->next, renderer_num++)
 		{
 			Renderer *renderer;
+			GdkRectangle cell_area;
 			gint width;
-			GtkSourceGutterRendererState state;
 			gint xpad;
 			gint ypad;
 
-			renderer = item->data;
-			width = g_array_index (sizes, gint, idx);
+			renderer = l->data;
+			width = g_array_index (renderer_widths, gint, renderer_num);
 
 			if (!gtk_source_gutter_renderer_get_visible (renderer->renderer))
 			{
+				g_assert_cmpint (width, ==, 0);
 				continue;
 			}
 
@@ -1143,22 +1171,8 @@ on_view_draw (GtkSourceView   *view,
 			cell_area.x = background_area.x + xpad;
 			cell_area.width = background_area.width - 2 * xpad;
 
-			state = GTK_SOURCE_GUTTER_RENDERER_STATE_NORMAL;
-
-			if (line_to_paint == cur_line)
-			{
-				state |= GTK_SOURCE_GUTTER_RENDERER_STATE_CURSOR;
-			}
-
-			if (has_selection &&
-			    gtk_text_iter_in_range (&start,
-			                            &selection_start,
-			                            &selection_end))
-			{
-				state |= GTK_SOURCE_GUTTER_RENDERER_STATE_SELECTED;
-			}
-
-			if (renderer->prelit >= 0 && cell_area.y <= renderer->prelit && cell_area.y + cell_area.height >= renderer->prelit)
+			if (renderer->prelit >= 0 &&
+			    cell_area.y <= renderer->prelit && renderer->prelit <= cell_area.y + cell_area.height)
 			{
 				state |= GTK_SOURCE_GUTTER_RENDERER_STATE_PRELIT;
 			}
@@ -1186,32 +1200,106 @@ on_view_draw (GtkSourceView   *view,
 			cairo_restore (cr);
 
 			background_area.x += background_area.width;
+			state &= ~GTK_SOURCE_GUTTER_RENDERER_STATE_PRELIT;
 		}
 
+		i++;
 		gtk_text_iter_forward_line (&start);
 	}
+}
 
-	/* Allow to call queue_redraw() in ::end. */
-	gutter->priv->is_drawing = FALSE;
+static void
+end_draw (GtkSourceGutter *gutter)
+{
+	GList *l;
 
-	for (item = gutter->priv->renderers; item != NULL; item = g_list_next (item))
+	for (l = gutter->priv->renderers; l != NULL; l = l->next)
 	{
-		Renderer *renderer = item->data;
+		Renderer *renderer = l->data;
 
 		if (gtk_source_gutter_renderer_get_visible (renderer->renderer))
 		{
 			gtk_source_gutter_renderer_end (renderer->renderer);
 		}
 	}
+}
 
-	g_array_free (numbers, TRUE);
-	g_array_free (pixels, TRUE);
-	g_array_free (heights, TRUE);
-	g_array_free (sizes, TRUE);
+static gboolean
+on_view_draw (GtkSourceView   *view,
+              cairo_t         *cr,
+              GtkSourceGutter *gutter)
+{
+	GdkRectangle clip;
+	GtkTextView *text_view;
+	gint first_y_window_coord;
+	gint last_y_window_coord;
+	gint first_y_buffer_coord;
+	gint last_y_buffer_coord;
+	GArray *renderer_widths;
+	LinesInfo *info;
+	GtkStyleContext *style_context;
+
+	if (!get_clip_rectangle (gutter, view, cr, &clip))
+	{
+		return GDK_EVENT_PROPAGATE;
+	}
+
+	gutter->priv->is_drawing = TRUE;
+
+	renderer_widths = g_array_new (FALSE, FALSE, sizeof (gint));
+	calculate_gutter_size (gutter, renderer_widths);
+
+	text_view = GTK_TEXT_VIEW (view);
+
+	first_y_window_coord = clip.y;
+	last_y_window_coord = first_y_window_coord + clip.height;
+
+	/* get the extents of the line printing */
+	gtk_text_view_window_to_buffer_coords (text_view,
+	                                       gutter->priv->window_type,
+	                                       0,
+	                                       first_y_window_coord,
+	                                       NULL,
+	                                       &first_y_buffer_coord);
+
+	gtk_text_view_window_to_buffer_coords (text_view,
+	                                       gutter->priv->window_type,
+	                                       0,
+	                                       last_y_window_coord,
+	                                       NULL,
+	                                       &last_y_buffer_coord);
+
+	info = get_lines_info (text_view,
+			       first_y_buffer_coord,
+			       last_y_buffer_coord);
+
+	style_context = gtk_widget_get_style_context (GTK_WIDGET (view));
+	gtk_style_context_save (style_context);
+	apply_style (gutter, view, style_context, cr);
+
+	begin_draw (gutter,
+		    text_view,
+		    renderer_widths,
+		    info,
+		    cr);
+
+	draw_cells (gutter,
+		    text_view,
+		    renderer_widths,
+		    info,
+		    cr);
+
+	/* Allow to call queue_redraw() in ::end. */
+	gutter->priv->is_drawing = FALSE;
+
+	end_draw (gutter);
 
 	gtk_style_context_restore (style_context);
 
-	return FALSE;
+	g_array_free (renderer_widths, TRUE);
+	lines_info_free (info);
+
+	return GDK_EVENT_PROPAGATE;
 }
 
 static Renderer *
