@@ -21,14 +21,13 @@
 
 #include "gtksourcestyleschememanager.h"
 #include "gtksourcestylescheme.h"
-#include "gtksourceview-marshal.h"
 #include "gtksourceview-i18n.h"
 #include "gtksourceview-utils.h"
 #include <string.h>
 
 /**
  * SECTION:styleschememanager
- * @Short_description: Object which provides access to GtkSourceStyleSchemes
+ * @Short_description: Provides access to GtkSourceStyleSchemes
  * @Title: GtkSourceStyleSchemeManager
  * @See_also: #GtkSourceStyleScheme
  *
