@@ -25,10 +25,6 @@
 #define __GTK_SOURCE_VIEW_H__
 
 #include <gtk/gtk.h>
-
-#include <gtksourceview/gtksourcebuffer.h>
-#include <gtksourceview/gtksourcegutter.h>
-#include <gtksourceview/gtksourcemarkattributes.h>
 #include <gtksourceview/gtksourcetypes.h>
 
 G_BEGIN_DECLS
@@ -129,13 +125,7 @@ typedef enum
 	GTK_SOURCE_DRAW_SPACES_LEADING    = 1 << 4,
 	GTK_SOURCE_DRAW_SPACES_TEXT       = 1 << 5,
 	GTK_SOURCE_DRAW_SPACES_TRAILING   = 1 << 6,
-	GTK_SOURCE_DRAW_SPACES_ALL        = (GTK_SOURCE_DRAW_SPACES_SPACE   | \
-	                                     GTK_SOURCE_DRAW_SPACES_TAB     | \
-	                                     GTK_SOURCE_DRAW_SPACES_NEWLINE | \
-	                                     GTK_SOURCE_DRAW_SPACES_NBSP | \
-	                                     GTK_SOURCE_DRAW_SPACES_LEADING | \
-	                                     GTK_SOURCE_DRAW_SPACES_TEXT | \
-	                                     GTK_SOURCE_DRAW_SPACES_TRAILING)
+	GTK_SOURCE_DRAW_SPACES_ALL        = 0x7f
 } GtkSourceDrawSpacesFlags;
 
 
@@ -174,7 +164,7 @@ gboolean	 gtk_source_view_get_indent_on_tab 	(GtkSourceView   *view);
 
 void		 gtk_source_view_set_highlight_current_line
 							(GtkSourceView   *view,
-							 gboolean         hl);
+							 gboolean         highlight);
 gboolean 	 gtk_source_view_get_highlight_current_line
 							(GtkSourceView   *view);
 
